# Foundation Auth

[![lerna](https://img.shields.io/badge/maintained%20with-lerna-cc00ff.svg)](https://lerna.js.org/)
[![TypeScript](https://img.shields.io/badge/%3C%2F%3E-TypeScript-%230074c1.svg)](https://www.typescriptlang.org/)

`foundation-auth` provides several exports to promote reuse and extensibility at various levels. These include but are
not limited to:

* `Auth`: The fully featured authentication micro frontend.
* `User`: The user singleton, which always exists in either an anonymous or authenticated mode.
* `AuthStore`: An auth based `foundation-store`.
* `AuthMachine`: An auth based `foundation-state-machine` covering the various auth workflows.
* `AuthMessageMapper`: The auth message dto to entity mapper.
* `LogoutUtility`: A logout utility, which you can call `run()` on to trigger a logout.
* Various UI components / screens.

In most cases, developers simply import both the `User` and `Auth` micro frontend, and use the `configure` export token
to tailor the micro frontend to their specific needs in their router config. However, developers can also use the
exports provided to build their own authentication frontends or workflows. A set of pre-defined custom events are used
to interact with the `AuthStore` and `AuthMachine`, and developers can provide custom implementations of parts of the
machine's workflow.

## Basic setup instructions
These examples are based on a host application using [FAST](https://www.fast.design/) and the
[FASTRouter](https://www.npmjs.com/package/@microsoft/fast-router). However, setup in the frontend and router of your
choice should be quite similar. 

## Environment variables
`foundation-auth` expects the following environment variables in order to operate. These could be loaded via a `.env`
file, but how you set them is up to you. Below is an example pointing to one of our test servers.

```dotenv
# HTTP
GENX_LOGIN_URL=https://public-foundation-hardened1/sm/event-login-auth
GENX_LOGIN_REFRESH_URL=https://public-foundation-hardened1/sm/event-login-auth
GENX_LOGIN_DETAILS_URL=https://public-foundation-hardened1/sm/event-login-details
GENX_LOGOUT_URL=https://public-foundation-hardened1/sm/event-logout
GENX_CHANGE_PASSWORD_URL=https://public-foundation-hardened1/sm/event-change-user-password
GENX_FORGOT_PASSWORD_URL=https://public-foundation-hardened1/sm/event-self-service-password-reset
GENX_RESET_PASSWORD_URL=https://public-foundation-hardened1/sm/event-password-reset-action

# Socket
GENX_SOCKET_URL=wss://public-foundation-hardened1/gwf/
```

### Get User reference
User is a singleton. You can access it via the DI container.
```ts
import { User } from '@genesislcap/foundation-auth/user';
...
@User private user: User
```

...or via a utility if your app is not using decorators or the DI container.
```ts
import { getUser, User } from '@genesislcap/foundation-auth/user';
...
private user: User = getUser();
```

User properties are decorated with `@observable`, meaning in FAST terms it's reactive, and you can bind to properties
directly. For example, `this.user.isAuthenticated`. However, you can also monitor the user's authentication status using
an event listener.

```ts
import type { UserAuthenticationChangeEvent } from '@genesislcap/foundation-auth/user';
...
this.user.addAuthenticationChangeListener((event: UserAuthenticationChangeEvent) => {
  console.log('User authenticated?', event.detail);
});
```

See the [User API Docs](./docs/api/foundation-auth.user.md) for more information.

### Set up a route to host the micro frontend
Here we are lazy loading the micro frontend when the route is accessed.
```ts
{
  title: 'Login',
  name: 'login',
  path: 'login', // < at what path should the micro frontend be accessed
  element: async () => {
    const { Auth, configure } = await import('@genesislcap/foundation-auth');
    configure({
      omitRoutes: ['request-account'],
      hostPath: 'login', // < important to match the route path
      defaultRedirectUrl: 'my-dashboard',
      logo: myLogo,
      background: myBG,
    });
    return Auth;
  },
  childRouters: true,
  settings: { public: true },
  layout: authLayout,
}
```

### Checking the user authentication status during routing
Here in our `FASTRouter`'s `NavigationContributor`, we check the user status as part of the routing flow.

```ts
this.contributors.push({
  navigate: async (phase) => {
    const settings = phase.route.settings;
    /**
     * Don't block public routes or any routes when authenticated
     */
    if (settings?.public || this.user.isAuthenticated) {
      return;
    }
    /**
     * Otherwise route them to login
     */
    phase.cancel(() => {
      this.user.trackPath();
      Route.name.replace(phase.router, 'login');
    });
  },
});
```

### Establishing a Websocket Connection
Post login you can import the `SOCKET_URL` from auth and connect to the backend. Below uses `connect` from the DI container.

```ts
import { SOCKET_URL } from '@genesislcap/foundation-auth/env';
...
@Connect private connect: Connect,
...
this.connect.connect(SOCKET_URL);
```

### Logging out
LogoutUtility is a singleton. You can access it via the DI container.
```ts
import { LogoutUtility } from '@genesislcap/foundation-auth/logout';
...
@LogoutUtility private logout: LogoutUtility;
...
this.logout.run();
```

...or via a utility if your app is not using decorators or the DI container.
```ts
import { getLogoutUtility, LogoutUtility } from '@genesislcap/foundation-auth/logout';
...
private logout: LogoutUtility = getLogoutUtility();
...
this.logout.run();
```

See the [LogoutUtility API Docs](./docs/api/foundation-auth.logoututility.md) for more information.

## [API Docs](./docs/api/index.md)

## License

Note: this project provides front-end dependencies and uses licensed components listed in the next section; thus, licenses for those components are required during development. Contact [Genesis Global](https://genesis.global/contact-us/) for more details.

### Licensed components
Genesis low-code platform
