import type { BasicCredentials, CredentialData, GetCredentialOptions, MFACredentials, RefreshCredentials, SSOCredentials } from './types';
import { CredentialManagerConfig } from './types';
/**
 * The default separator used for splitting credential parts.
 * @public
 */
export declare const credentialSeparator = ":";
/**
 * The default request options for the CredentialManager service.
 * @public
 */
export declare const defaultCredentialRequestOptions: GetCredentialOptions;
/**
 * The default configuration for the CredentialManager service.
 * @public
 */
export declare const defaultCredentialManagerConfig: CredentialManagerConfig;
/**
 * CredentialManager interface.
 *
 * @remarks
 * A system for managing user credentials.
 *
 * @public
 */
export interface CredentialManager {
    /**
     * Validates basic credentials (username and password).
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateBasicCredentials(credentials: BasicCredentials): boolean;
    /**
     * Validates refresh credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateRefreshCredentials(credentials: RefreshCredentials): boolean;
    /**
     * Validates SSO credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateSSOCredentials(credentials: SSOCredentials): boolean;
    /**
     * Validates MFA credentials.
     * @public
     * @param credentials - The credentials to validate.
     * @returns `true` if the credentials are valid, or `false` otherwise.
     */
    validateMFACredentials(credentials: MFACredentials): boolean;
    /**
     * Validate user credential data returned from the container or a cookie.
     *
     * @remarks
     * This can be called to ensure the credential is in the format this version of the manager expects. This is mainly
     * for dealing with malformed cookies.
     *
     * @privateRemarks
     * This can occur if the credential was saved using custom App code or via an early version of the manager itself.
     * The credential may be missing certain field values.
     *
     * @public
     * @param data - The credential data to validate.
     * @param ensurePassword - Check for the existence of a password. PasswordCredentials will have this, but cookies are considered valid without.
     */
    validateUserCredentialData(data: CredentialData, ensurePassword?: boolean): boolean;
    /**
     * Normalise legacy credential data.
     *
     * @remarks
     * Older applications may have legacy credential data that needs migrating to the latest format the manager expects.
     * If the input CredentialData is already formatted as such, it will be returned as is. Otherwise, we check if any
     * cookie input is already formatted as such, and if so return it as CredentialData. If neither inputs are valid,
     * we will attempt a best effort normalisation to create valid CredentialData, or abort and return undefined.
     *
     * @public
     * @param data - The credential data to normalise.
     * @param cookie - The unknown cookie data to normalise.
     * @returns A tuple of the best effort normalisation of the inputs to the CredentialData format, and a boolean to indicate if we can remove the cookie.
     */
    normaliseLegacyCredentialData(data?: CredentialData, cookie?: CredentialData): [CredentialData | undefined, boolean];
    /**
     * Stores the specified credential data in the browser's credential manager.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CredentialsContainer
     * @public
     * @param data - The credential data to store.
     * @returns A Promise that resolves to the stored credential object, or `undefined` if storage fails.
     */
    storeCredentialsInContainer(data: CredentialData): Promise<Credential | undefined>;
    /**
     * Retrieves credential data from the browser's credential manager.
     * @public
     * @param options - Options for filtering the retrieved credentials, if any.
     * @returns A Promise that resolves to the retrieved credential data, or `null` if no matching credentials were found.
     */
    getCredentialsFromContainer(options?: GetCredentialOptions): Promise<CredentialData | null>;
    /**
     * Stores the specified credential data in a browser cookie.
     * @public
     * @param data - The credential data to store.
     * @returns `true` if storage was successful, or `false` otherwise.
     */
    storeCredentialsInCookie(data: CredentialData): boolean;
    /**
     * Retrieves credential data from a browser cookie.
     * @public
     * @returns The retrieved credential data, or `undefined` if no matching data was found.
     */
    getCredentialsFromCookie(): CredentialData | undefined;
    /**
     * Stores user credentials securely.
     * @public
     * @param data - The credential data to be stored.
     * @returns A Promise that resolves to a boolean indicating if the credentials were stored successfully.
     */
    storeCredentials(data: CredentialData): Promise<boolean>;
    /**
     * Retrieves stored credentials.
     * @public
     * @param options - An object containing options for retrieving credentials.
     * @returns A Promise that resolves to a CredentialData object if the credentials are found, or undefined otherwise.
     */
    getCredentials(options?: GetCredentialOptions): Promise<CredentialData | undefined>;
    /**
     * Removes the user's credential cookie.
     * @remarks
     * This is generally called when migrating a user's saved credentials from a cookie to the secure container.
     * @public
     */
    removeCookie(): void;
    /**
     * Prevents silent access by requiring the user to authenticate before making API requests.
     * @public
     */
    preventSilentAccess(): Promise<void>;
}
/**
 * Default CredentialManager.
 * @public
 */
export declare class DefaultCredentialManager implements CredentialManager {
    private config;
    constructor(config?: CredentialManagerConfig);
    validateBasicCredentials(credentials: BasicCredentials): boolean;
    validateRefreshCredentials(credentials: RefreshCredentials): boolean;
    validateSSOCredentials(credentials: SSOCredentials): boolean;
    validateMFACredentials(credentials: MFACredentials): boolean;
    validateUserCredentialData(data: CredentialData, ensurePassword?: boolean): boolean;
    normaliseLegacyCredentialData(data?: CredentialData, cookie?: CredentialData): [CredentialData | undefined, boolean];
    storeCredentialsInContainer(data: CredentialData): Promise<Credential | undefined>;
    getCredentialsFromContainer(options?: CredentialRequestOptions): Promise<CredentialData | null>;
    storeCredentialsInCookie(data: CredentialData): boolean;
    getCredentialsFromCookie(): CredentialData | undefined;
    storeCredentials(data: CredentialData): Promise<boolean>;
    getCredentials(options?: GetCredentialOptions): Promise<CredentialData | undefined>;
    removeCookie(): void;
    preventSilentAccess(): Promise<void>;
}
/**
 * CredentialManager DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const CredentialManager: import("@microsoft/fast-foundation").InterfaceSymbol<CredentialManager>;
//# sourceMappingURL=credentialManager.d.ts.map