import type { MachineContext } from '@genesislcap/foundation-state-machine';
import type { DoneInvokeEvent, ErrorEvent } from 'xstate';
import { Credentials } from '../credential';
import type { AuthDTO, AuthEntity } from '../mapper';
import type { User } from '../user';
/**
 * @public
 */
export type LoginParams = Credentials;
/**
 * @public
 */
export type ChangePasswordParams = {
    username: string;
    oldPassword: string;
    newPassword: string;
    requester?: string;
};
/**
 * @public
 */
export type ForgotPasswordParams = {
    username: string;
    returnUrl: string;
    requester?: string;
};
/**
 * @public
 */
export type ResetPasswordParams = {
    username: string;
    resetToken: string;
    newPassword: string;
    requester?: string;
};
/**
 * @public
 */
export type QueryParams = {
    SSO_TOKEN?: string;
    MFA_CODE?: string;
};
/**
 * AuthMachineEventParams type.
 * @public
 */
export type AuthMachineEventParams = LoginParams | ForgotPasswordParams | ChangePasswordParams | ResetPasswordParams | QueryParams;
/**
 * AuthMachineContext interface.
 * @remarks
 * Transient state during the machine's execution.
 * @public
 */
export interface AuthMachineContext extends MachineContext {
    /**
     * Authentication event params.
     */
    params?: AuthMachineEventParams;
    /**
     * The login/login DTO from the fetch machine.
     */
    dto: AuthDTO;
    /**
     * The login/login Entity mapped from the DTO.
     */
    entity: AuthEntity;
    /**
     * The user, either authenticated or anonymous.
     */
    user: User;
}
/**
 * AuthMachineEvent type.
 * @remarks
 * The send action events used to invoke the machine.
 * @public
 */
export type AuthMachineEvent = {
    type: 'login';
    params: LoginParams;
} | {
    type: 'logout';
} | {
    type: 'change-password';
    params: ChangePasswordParams;
} | {
    type: 'forgot-password';
    params: ForgotPasswordParams;
} | {
    type: 'reset-password';
    params: ResetPasswordParams;
} | DoneInvokeEvent<unknown> | ErrorEvent<unknown>;
/**
 * AuthMachineEventMap type.
 * @remarks
 * The mediation CustomEvent used to invoke machine actions.
 * @public
 */
export type AuthMachineEventMap = {
    'auth-machine-event': AuthMachineEvent;
};
/**
 * Default AuthMachineContext.
 * @public
 */
export declare const defaultAuthMachineContext: Partial<AuthMachineContext>;
/**
 * isLoginParams.
 * @param params - Potential LoginParams.
 * @public
 */
export declare function isLoginParams(params: AuthMachineEventParams): params is LoginParams;
/**
 * isForgotPasswordParams.
 * @remarks
 * ForgotPasswordParams.requester is optional, so it is not part of the evaluation.
 * @param params - Potential ForgotPasswordParams.
 * @public
 */
export declare function isForgotPasswordParams(params: AuthMachineEventParams): params is ForgotPasswordParams;
/**
 * isChangePasswordParams.
 * @remarks
 * ChangePasswordParams.requester is optional, so it is not part of the evaluation.
 * @param params - Potential ChangePasswordParams.
 * @public
 */
export declare function isChangePasswordParams(params: AuthMachineEventParams): params is ChangePasswordParams;
/**
 * isResetPasswordParams.
 * @remarks
 * ResetPasswordParams.requester is optional, so it is not part of the evaluation.
 * @param params - Potential ResetPasswordParams.
 * @public
 */
export declare function isResetPasswordParams(params: AuthMachineEventParams): params is ResetPasswordParams;
/**
 * isQueryParams.
 * @param params - Potential QueryParams.
 * @public
 */
export declare function isQueryParams(params: AuthMachineEventParams): params is QueryParams;
//# sourceMappingURL=types.d.ts.map