import { Mapper } from './types';
/**
 * @public
 */
export type MessageMapperLookup = Record<string, Mapper<unknown, unknown>>;
/**
 * @public
 */
export interface MessageMapper<TLookup extends MessageMapperLookup, K extends keyof TLookup = keyof TLookup> extends Pick<Map<K, TLookup[K]>, 'get' | 'has' | 'delete' | 'clear' | 'size'> {
    /**
     * Set a mapper by message type.
     * @param key - The message type as key.
     * @param mapper - The mapper.
     *
     * @public
     */
    set(key: K, mapper: TLookup[K]): void;
}
/**
 * @public
 */
export declare class DefaultMessageMapper<TLookup extends MessageMapperLookup, K extends keyof TLookup = keyof TLookup> implements MessageMapper<TLookup, K> {
    private map;
    /** {@inheritDoc MessageMapper.set} */
    set(key: K, mapper: TLookup[K]): void;
    /**
     * Get a mapper by message type.
     * @param key - The message type as key.
     * @returns The mapper if it exists, otherwise undefined.
     * @public
     */
    get(key: K): TLookup[K] | undefined;
    /**
     * Has a mapper for key.
     * @param key - The message type as key.
     * @returns True if it exists, otherwise false.
     *
     * @public
     */
    has(key: K): boolean;
    /**
     * Delete a mapper.
     * @param key - The message type as key.
     * @returns True if successful, otherwise false.
     *
     * @public
     */
    delete(key: K): boolean;
    /**
     * Clear mappers.
     * @public
     */
    clear(): void;
    /**
     * The size of the mapper map.
     * @returns number
     *
     * @public
     */
    get size(): number;
}
/**
 * A factory to create a MessageMapper.
 * @returns A MessageMapper instance.
 *
 * @public
 */
export declare const createMessageMapper: <TLookup extends MessageMapperLookup, K extends keyof TLookup = keyof TLookup>() => MessageMapper<TLookup, K>;
//# sourceMappingURL=map.d.ts.map