import { Listener } from '@genesislcap/foundation-utils';
import type { Details, LoginAck } from '../entity/loginAck';
import { UserConfig } from './config';
/**
 * UserAuthenticationChangeType.
 * @public
 */
export declare const UserAuthenticationChangeType = "user-authentication-change";
/**
 * UserAuthenticationChangeEvent.
 *
 * @remarks
 * Detail will either be `true` or `false` to indicate authentication status.
 *
 * @public
 */
export type UserAuthenticationChangeEvent = CustomEvent<boolean>;
/**
 * UserPathHistory.
 *
 * @remarks
 * APIs to track a user's path history. Includes the pathname, search and hash only.
 * This is not intended proxy or replace the History API.
 *
 * @privateRemarks
 * Integrate with foundation analytics? Users could then simply call `user.trackPath` and we could engage analytics.
 * We should move analytics out of comms first.
 *
 * @public
 */
export interface UserPathHistory {
    /**
     * Array of paths that have been tracked.
     */
    readonly pathHistory: string[];
    /**
     * Track current path.
     */
    trackPath(): string;
    /**
     * Clears path history.
     */
    clearPaths(): void;
    /**
     * May be used to return to a target path after authentication.
     * @returns The last stored path.
     */
    lastPath(): string | undefined;
}
/**
 * UserActionHistory.
 *
 * @remarks
 * APIs to track a user's action history.
 *
 * @privateRemarks
 * Integrate with foundation analytics? Users could then simply call `user.trackAction` and we could engage analytics.
 * We should move analytics out of comms first.
 *
 * @public
 */
export interface UserActionHistory {
    /**
     * Array of actions that have been tracked.
     */
    readonly actionHistory: unknown[];
    /**
     * Track an action.
     */
    trackAction<TDetails = unknown>(details: TDetails): void;
    /**
     * Clears action history.
     */
    clearActions(): void;
    /**
     * @returns The last stored action.
     */
    lastAction(): unknown;
}
/**
 * User interface.
 *
 * @remarks
 * The user singleton, which always exists in either an anonymous or authenticated mode.
 *
 * @privateRemarks
 * Providing `username` and `userName` for backwards compatability.
 *
 * @example
 * ```ts
 * import { User } from '@genesislcap/foundation-auth/user';
 * ...
 * @User private user: User
 * ```
 *
 * @public
 */
export interface User extends UserPathHistory, UserActionHistory {
    readonly isAuthenticated: boolean;
    readonly isAnonymous: boolean;
    readonly userKVs: {
        [key: string]: any;
    };
    readonly details: Details;
    readonly permission: string[];
    readonly profile: string[];
    readonly sourceRef: string;
    readonly userName: string;
    readonly username: string;
    /**
     * @param entity - The LoginAck used to populate the User.
     */
    set(entity: Partial<LoginAck>): this;
    unset(): this;
    isAdmin(): boolean;
    isBanned(): boolean;
    /**
     * @param permission - A permission string or an array of permission strings to check.
     */
    hasPermission(permission: string | string[]): boolean;
    /**
     * @param profile - A profile string or an array of profile strings to check.
     */
    hasProfile(profile: string | string[]): boolean;
    /**
     * @example
     * ```ts
     * import type { UserAuthenticationChangeEvent } from '@genesislcap/foundation-auth/user';
     * ...
     * this.user.addAuthenticationChangeListener((event: UserAuthenticationChangeEvent) => {
     *   console.log('User authenticated?', event.detail);
     * });
     * ```
     *
     * @param listener - A listener function.
     */
    addAuthenticationChangeListener(listener: Listener<UserAuthenticationChangeEvent>): void;
}
/**
 * Default User implementation.
 * @public
 */
export declare class DefaultUser implements User {
    /** @internal */
    protected config: UserConfig;
    /** {@inheritDoc UserPathHistory.pathHistory} */
    pathHistory: string[];
    /** {@inheritDoc UserActionHistory.actionHistory} */
    actionHistory: unknown[];
    /** {@inheritDoc User.isAuthenticated} */
    isAuthenticated: boolean;
    /** {@inheritDoc User.userKVs} */
    userKVs: {
        [key: string]: any;
    };
    /** {@inheritDoc User.details} */
    details: Details;
    /** {@inheritDoc User.permission} */
    permission: string[];
    /** {@inheritDoc User.profile} */
    profile: string[];
    /** {@inheritDoc User.sourceRef} */
    sourceRef: string;
    /** {@inheritDoc User.userName} */
    userName: string;
    /** {@inheritDoc User.set} */
    set(entity: Partial<LoginAck>): this;
    /** {@inheritDoc User.unset} */
    unset(): this;
    /** @internal */
    protected userNameChanged(): void;
    /** {@inheritDoc User.username} */
    get username(): string;
    /** {@inheritDoc User.isAnonymous} */
    get isAnonymous(): boolean;
    /** {@inheritDoc User.isAdmin} */
    isAdmin(): boolean;
    /** {@inheritDoc User.isBanned} */
    isBanned(): boolean;
    /** {@inheritDoc User.hasPermission} */
    hasPermission(permission: string | string[]): boolean;
    /** {@inheritDoc User.hasProfile} */
    hasProfile(profile: string | string[]): boolean;
    /** {@inheritDoc UserPathHistory.trackPath} */
    trackPath(): string;
    /** {@inheritDoc UserPathHistory.clearPaths} */
    clearPaths(): void;
    /** {@inheritDoc UserPathHistory.lastPath} */
    lastPath(): string;
    /** {@inheritDoc UserActionHistory.trackAction} */
    trackAction<TDetails = unknown>(details: TDetails): void;
    /** {@inheritDoc UserActionHistory.clearActions} */
    clearActions(): void;
    /** {@inheritDoc UserActionHistory.lastAction} */
    lastAction(): unknown;
    /** @internal */
    protected hasKey(keys: string[], key: string | string[]): boolean;
    /** @internal */
    protected authenticationChangeListeners: import("@genesislcap/foundation-utils").Observer<UserAuthenticationChangeEvent>;
    /** {@inheritDoc User.addAuthenticationChangeListener} */
    addAuthenticationChangeListener(listener: Listener<UserAuthenticationChangeEvent>): () => void;
    /** @internal */
    protected isAuthenticatedChanged(): void;
}
/**
 * User DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const User: import("@microsoft/fast-foundation").InterfaceSymbol<User>;
/**
 * Gets User from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getUser, User } from '@genesislcap/foundation-auth/user';
 * ...
 * private user: User = getUser();
 * ```
 *
 * @public
 */
export declare function getUser(): User;
//# sourceMappingURL=user.d.ts.map