import { css } from '@microsoft/fast-element';
import { DI, Registration } from '@microsoft/fast-foundation';
import { Route } from '@microsoft/fast-router';
import defaultBackgroundSrc from '../assets/bg.jpg';
import defaultLogoSrc from '../assets/genesis-logo-light.svg';
/**
 * @public
 */
export const Routes = {
    login: 'login',
    logout: 'logout',
    'request-account': 'request-account',
    'reset-password': 'reset-password',
    'forgot-password': 'forgot-password',
    verify: 'verify',
    protected: 'protected',
    'not-found': 'not-found',
};
/**
 * A utility to check if a pathname is a route of this micro frontend.
 *
 * @param pathname - A pathname string to check, typically location.pathname, which is used when a value is omitted.
 *
 * @public
 * @privateRemarks
 * I've just put this util in for now given there's some app level direction code in place which needs review.
 */
export function isMFRoute(pathname = location.pathname) {
    const valueParts = pathname.split('/');
    const routeIdx = pathname.charAt(0) === '/' ? 1 : 0;
    const route = valueParts[routeIdx];
    return !!Routes[route];
}
/**
 * A utility to get a sub route path taking the host path into account.
 *
 * @param routeName - The route name.
 * @param hostPath - The host path.
 *
 * @public
 */
export function getSubRoutePath(routeName, hostPath = '') {
    return hostPath !== '' ? `${hostPath}/${routeName}` : routeName;
}
/**
 * Default AuthConfig DI implementation.
 * @public
 */
export const defaultAuthConfig = {
    logo: css `
    content: url(${defaultLogoSrc});
  `,
    background: css `
    :host {
      background-image: url(${defaultBackgroundSrc});
    }
  `,
    backgroundVideo: null,
    omitRoutes: [],
    fields: {
        organisation: null,
        username: {
            label: 'Username',
            pattern: '^[a-zA-Z0-9.@_\\-]*$',
            title: 'Letters, numbers, and limited special characters @ _ - .',
        },
        email: {
            label: 'Email Address',
        },
        password: {
            label: 'Password',
        },
    },
    hostPath: '',
    defaultRedirectUrl: '/protected',
    forgotPasswordReturnUrl: `${location.protocol}//${location.host}/${getSubRoutePath('reset-password')}`,
    omitRedirectUrls: ['/not-found', '/not-permitted'],
    routeToUrl: (url) => {
        if (!!url && url !== Route.path.current) {
            /**
             * Uses the FASTRouter by default, however, host apps may override to integrate with their router of choice.
             * The FASTRouter invokes the History API and triggers a NavigationMessage to drive the view change.
             * TODO: We will need to test how unknown app level routers react to this default when not overriding.
             */
            Route.path.push(url);
        }
    },
    sso: {
        identityProvidersPath: 'gwf/sso/list',
    },
    feedbackDelay: 3000,
};
/**
 * AuthConfig DI key.
 *
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export const AuthConfig = DI.createInterface((x) => x.instance(defaultAuthConfig));
/**
 * Configure the auth micro frontend settings.
 *
 * @remarks
 * This is primarily for general settings, rather than re-defining the element, static templating / style changes.
 *
 * @param config - A partial AuthConfig.
 *
 * @example Configuring auth on lazy load
 * ```ts
 * name: 'auth',
 * path: 'auth',
 * title: 'Authenticate',
 * element: async () => {
 *   const { configure, Auth } = await import('@genesislcap/foundation-auth');
 *   configure({
 *     hostPath: 'auth',
 *     defaultRedirectUrl: 'dashboard',
 *     omitRoutes: ['request-account'],
 *     logo: myCompanyLogo,
 *     background: myAuthBG,
 *   });
 *   return Auth;
 * },
 * layout: authLayout,
 * settings: { public: true },
 * childRouters: true,
 * ```
 *
 * @public
 * @privateRemarks
 * We can tailor the spreading logic after we put it to use. Keeping it simple for now.
 */
export function configure(config) {
    DI.getOrCreateDOMContainer().register(Registration.instance(AuthConfig, Object.assign(Object.assign({}, defaultAuthConfig), config)));
}
