import { __decorate } from "tslib";
import { UUID } from '@genesislcap/foundation-utils';
import { DI } from '@microsoft/fast-foundation';
import { CredentialType } from '../credential';
import { MESSAGE_TYPE } from '../dto';
import { isLoginParams } from '../machine';
import { logger } from '../utils';
import { MappingError } from './error';
import { fromDTOAck, fromDTONack, fromDTOStringArray, hasDTOErrorWarningCode } from './utils';
/**
 * isLoginAckDTO.
 * @param dto - Potential DTO.
 * @public
 */
export function isLoginAckDTO(dto) {
    return (dto === null || dto === void 0 ? void 0 : dto.MESSAGE_TYPE) === MESSAGE_TYPE.LOGIN_ACK;
}
/**
 * isLoginDetailsAckDTO.
 * @param dto - Potential DTO.
 * @public
 */
export function isLoginDetailsAckDTO(dto) {
    return (dto === null || dto === void 0 ? void 0 : dto.MESSAGE_TYPE) === MESSAGE_TYPE.LOGIN_DETAILS_ACK;
}
/**
 * isLoginNackDTO.
 * @param dto - Potential DTO.
 * @public
 */
export function isLoginNackDTO(dto) {
    return (dto === null || dto === void 0 ? void 0 : dto.MESSAGE_TYPE) === MESSAGE_TYPE.LOGIN_NACK;
}
/**
 * isLoginAckEntity.
 * @param entity - Potential entity.
 * @public
 */
export function isLoginAckEntity(entity) {
    return (entity === null || entity === void 0 ? void 0 : entity.messageType) === MESSAGE_TYPE.LOGIN_ACK;
}
/**
 * isLoginDetailsAckEntity.
 * @param entity - Potential entity.
 * @public
 */
export function isLoginDetailsAckEntity(entity) {
    return (entity === null || entity === void 0 ? void 0 : entity.messageType) === MESSAGE_TYPE.LOGIN_DETAILS_ACK;
}
/**
 * isLoginNackEntity.
 * @param entity - Potential entity.
 * @public
 */
export function isLoginNackEntity(entity) {
    return (entity === null || entity === void 0 ? void 0 : entity.messageType) === MESSAGE_TYPE.LOGIN_NACK;
}
/**
 * Default LoginMapper.
 * @public
 */
export class DefaultLoginMapper {
    constructor() {
        /**
         * {@inheritDoc Mapper.toEntity}
         */
        this.toEntity = (dto) => {
            try {
                if (isLoginAckDTO(dto) || isLoginDetailsAckDTO(dto)) {
                    const { DETAILS } = dto;
                    const ack = Object.assign(Object.assign({}, fromDTOAck(dto)), { userName: dto.USER_NAME, updateQueue: dto.UPDATE_QUEUE, zeroMqProxyModeEnabled: dto.ZERO_MQ_PROXY_MODE_ENABLED, zeroMqProxyInboundPort: dto.ZERO_MQ_PROXY_INBOUND_PORT, zeroMqProxyOutboundPort: dto.ZERO_MQ_PROXY_OUTBOUND_PORT, userKVs: dto.USER, details: DETAILS
                            ? {
                                system: this.fromSystemDTO(DETAILS.SYSTEM),
                                heartbeatIntervalSecs: DETAILS.HEARTBEAT_INTERVAL_SECS,
                                failedLoginAttempts: DETAILS.FAILED_LOGIN_ATTEMPTS,
                                rejectedLoginAttempts: DETAILS.REJECTED_LOGIN_ATTEMPTS,
                                lastLoginDatetime: DETAILS.LAST_LOGIN_DATETIME,
                                daysToPasswordExpiry: DETAILS.DAYS_TO_PASSWORD_EXPIRY,
                                product: this.fromProductDTO(DETAILS.PRODUCT),
                            }
                            : null, permission: fromDTOStringArray(dto.PERMISSION), profile: fromDTOStringArray(dto.PROFILE) });
                    return ack;
                }
                if (isLoginNackDTO(dto)) {
                    const nack = Object.assign(Object.assign({}, fromDTONack(dto)), { refreshAuthToken: dto.REFRESH_AUTH_TOKEN, isMFARequired: hasDTOErrorWarningCode(dto, 'MFA_REQUIRED'), hasPasswordExpired: hasDTOErrorWarningCode(dto, 'PASSWORD_EXPIRED') });
                    return nack;
                }
                throw new Error('No Entity mapping implementation for provided DTO');
            }
            catch (e) {
                logger.error(e);
                throw new MappingError(dto);
            }
        };
        /**
         * {@inheritDoc Mapper.toDTO}
         */
        this.toDTO = (entity) => {
            try {
                const params = entity;
                if (isLoginParams(params)) {
                    let DETAILS;
                    switch (params.type) {
                        case CredentialType.BASIC:
                            DETAILS = {
                                USER_NAME: params.username,
                                PASSWORD: params.password,
                            };
                            break;
                        case CredentialType.REFRESH:
                            DETAILS = {
                                USER_NAME: params.username,
                                REFRESH_AUTH_TOKEN: params.refreshToken,
                            };
                            break;
                        case CredentialType.SSO:
                            DETAILS = {
                                SSO_TOKEN: params.ssoToken,
                            };
                            break;
                        case CredentialType.JWT:
                            DETAILS = {
                                JWT: params.jwt,
                                DOMAIN: params.domain,
                            };
                            break;
                        case CredentialType.MFA:
                            DETAILS = {
                                MFA_CODE: params.mfaToken,
                            };
                            break;
                    }
                    const dto = {
                        DETAILS,
                        SOURCE_REF: this.uuid.createId(),
                        MESSAGE_TYPE: MESSAGE_TYPE.LOGIN,
                    };
                    return dto;
                }
                throw new Error('No DTO mapping implementation for provided entity');
            }
            catch (e) {
                logger.error(e);
                throw new MappingError(entity);
            }
        };
        this.fromSystemDTO = (system) => system ? { date: system.DATE } : null;
        this.fromProductDTO = (products) => products
            ? products.map((p) => ({
                name: p.NAME,
                version: p.VERSION,
            }))
            : [];
    }
}
__decorate([
    UUID
], DefaultLoginMapper.prototype, "uuid", void 0);
/**
 * LoginMapper DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const LoginMapper = DI.createInterface((x) => x.singleton(DefaultLoginMapper));
