import { __awaiter, __decorate, __param } from "tslib";
import { Container } from '@microsoft/fast-foundation';
import { RouterConfiguration, Route } from '@microsoft/fast-router';
import { defaultLayout } from '../layouts';
import { AuthConfig } from '../main/config';
import { User } from '../user';
import { ForgotPassword } from './forgot-password/forgot-password';
import { Login } from './login/login';
import { Logout } from './logout/logout';
import { NotFound } from './not-found/not-found';
import { Protected } from './protected/protected';
import { RequestAccount } from './request-account/request-account';
import { ResetPassword } from './reset-password/reset-password';
import { Verify } from './verify/verify';
/**
 * Base implementation of a routing used with the Auth class
 *
 * @internal
 */
let MainRouterConfig = class MainRouterConfig extends RouterConfiguration {
    constructor(user, container, authConfig) {
        super();
        this.user = user;
        this.container = container;
        this.authConfig = authConfig;
    }
    configure() {
        this.title = 'Login';
        this.defaultLayout = defaultLayout;
        /**
         * TODO: Cascade the template options to child routes in a subsequent PR
         * element: async () => {
         *    const { define, templateOptions, template, styles } = await import(
         *        /!* webpackChunkName: "auth-login" *!/
         *        './auth-login'
         *    );
         *    return define(this.compose(templateOptions, template, styles, 'auth-login'));
         * },
         */
        const routes = [
            {
                path: '',
                name: 'login',
                element: Login,
                title: 'Login',
                settings: { public: true },
            },
            {
                path: 'logout',
                name: 'logout',
                element: Logout,
                title: 'Logging out, please wait...',
                settings: { public: true },
            },
            {
                path: 'not-found',
                element: NotFound,
                title: 'Not Found',
                name: 'not-found',
                settings: { public: true },
            },
        ];
        const { omitRoutes } = this.authConfig;
        if (!omitRoutes.includes('request-account')) {
            routes.push({
                path: 'request-account',
                name: 'request-account',
                element: RequestAccount,
                title: 'Request Account',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('reset-password')) {
            routes.push({
                path: 'reset-password',
                name: 'reset-password',
                element: ResetPassword,
                title: 'Reset Password',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('forgot-password')) {
            routes.push({
                path: 'forgot-password',
                name: 'forgot-password',
                element: ForgotPassword,
                title: 'Forgot Password',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('verify')) {
            routes.push({
                path: 'verify',
                element: Verify,
                title: 'Verify',
                name: 'verify',
                settings: { public: true },
            });
        }
        if (!omitRoutes.includes('protected')) {
            routes.push({
                path: 'protected',
                element: Protected,
                title: 'Protected',
                name: 'protected',
                settings: { public: false },
            });
        }
        this.routes.map(...routes);
        /**
         * Example of a FallbackRouteDefinition
         */
        this.routes.fallback(() => this.user.isAuthenticated ? { redirect: 'not-found' } : { redirect: 'login' });
        /**
         * Example of a NavigationContributor
         */
        this.contributors.push({
            navigate: (phase) => __awaiter(this, void 0, void 0, function* () {
                const settings = phase.route.settings;
                /**
                 * If public route don't block
                 */
                if (settings && settings.public) {
                    return;
                }
                /**
                 * If authenticated don't block
                 */
                if (this.user.isAuthenticated) {
                    return;
                }
                /**
                 * Otherwise route them to login
                 */
                phase.cancel(() => {
                    this.user.trackPath();
                    Route.name.replace(phase.router, 'login');
                });
            }),
        });
    }
    construct(Type) {
        return this.container.get(Type);
    }
};
MainRouterConfig = __decorate([
    __param(0, User),
    __param(1, Container),
    __param(2, AuthConfig)
], MainRouterConfig);
export { MainRouterConfig };
