import { __awaiter, __decorate } from "tslib";
import { inIFrame, inSymphonyDesktop, openPopup } from '@genesislcap/foundation-utils';
import { customElement, DOM, observable, volatile } from '@microsoft/fast-element';
import { CredentialType } from '../../credential';
import { fetchIDPs, getSSOLoginURL, logger } from '../../utils';
import { BaseRoute } from '../base';
import { LoginStyles as styles } from './login.styles';
import { LoginTemplate as template } from './login.template';
let Login = class Login extends BaseRoute {
    constructor() {
        super(...arguments);
        this.idps = [];
        this.selectedIDP = '';
    }
    connectedCallback() {
        super.connectedCallback();
        this.addStoreBindings();
    }
    enter() {
        /**
         * Retaining the logic that logs authenticated users out when this route is visited for now.
         * TODO: Revisit as shouldn't load mf if logged in already.
         */
        if (this.user.isAuthenticated) {
            this.logout();
        }
    }
    attemptBasicLogin() {
        const basicCredentials = {
            type: CredentialType.BASIC,
            username: this.store.formEntry.username,
            password: this.store.formEntry.password,
        };
        if (!this.credentialManager.validateBasicCredentials(basicCredentials)) {
            logger.debug(`Basic credentials appear invalid, aborting login attempt.`);
            return;
        }
        logger.debug(`Attempting to login with basic credentials.`);
        this.login(basicCredentials);
    }
    // TODO: Move this out...
    addStoreBindings() {
        this.store.formEntry.binding((x) => x.ssoToggled, (value) => {
            var _a, _b;
            this.clearError();
            if (value) {
                const path = `${(_b = (_a = this.config.sso) === null || _a === void 0 ? void 0 : _a.identityProvidersPath) !== null && _b !== void 0 ? _b : 'sso/list'}`;
                fetchIDPs(path)
                    .then((idps) => {
                    var _a, _b, _c;
                    logger.debug('fetched IDPs', idps);
                    // it no longer throws, check for 'error' manually
                    if (!idps.length || (idps.length === 1 && ((_a = idps[0]) === null || _a === void 0 ? void 0 : _a.id) === 'error')) {
                        this.setError('Failed to fetch identity providers.');
                    }
                    else {
                        // Move to store, machine context, or keep local?
                        this.idps = idps;
                        this.selectedIDP = (_c = (_b = this.idps) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id;
                    }
                })
                    .catch(() => this.setError('Failed to fetch identity providers.'));
            }
        });
    }
    get selectedIDPName() {
        return this.selectedIDP ? ` - ${this.selectedIDP}` : '';
    }
    onClickSSO() {
        const selectedIDP = this.idps.find((idp) => idp.id === this.selectedIDP);
        const ssoLoginUrl = getSSOLoginURL(selectedIDP);
        if (inIFrame()) {
            if (inSymphonyDesktop()) {
                // symphony/electron opens new window in the default browser instead
                // popup within electron, so we have to navigate manually
                openPopup('', 'symphony').location.assign(ssoLoginUrl);
            }
            else {
                openPopup(ssoLoginUrl, 'Login');
            }
        }
        else {
            window.open(ssoLoginUrl, '_self');
        }
    }
    onLoginSubmit() {
        return __awaiter(this, void 0, void 0, function* () {
            DOM.queueUpdate(() => {
                this.attemptBasicLogin();
            });
        });
    }
    login(params) {
        this.$emit('auth-login', params);
    }
    logout() {
        this.$emit('auth-logout');
    }
    get submitDisabled() {
        return this.store.authMachine.hasTag('pending');
    }
};
__decorate([
    observable
], Login.prototype, "idps", void 0);
__decorate([
    observable
], Login.prototype, "selectedIDP", void 0);
__decorate([
    volatile
], Login.prototype, "selectedIDPName", null);
Login = __decorate([
    customElement({
        name: 'foundation-auth-login',
        template,
        styles,
    })
], Login);
export { Login };
