import { __decorate } from "tslib";
import { createObserver } from '@genesislcap/foundation-utils';
import { Observable, observable } from '@microsoft/fast-element';
import { DI } from '@microsoft/fast-foundation';
import { UserConfig } from './config';
/**
 * UserAuthenticationChangeType.
 * @public
 */
export const UserAuthenticationChangeType = 'user-authentication-change';
/**
 * Default User implementation.
 * @public
 */
export class DefaultUser {
    constructor() {
        /** {@inheritDoc UserPathHistory.pathHistory} */
        this.pathHistory = [];
        /** {@inheritDoc UserActionHistory.actionHistory} */
        this.actionHistory = [];
        /** @internal */
        this.authenticationChangeListeners = createObserver();
    }
    /** {@inheritDoc User.set} */
    set(entity) {
        if (this.isAuthenticated) {
            throw new Error(`Only AuthMachine should call User.set. If this is intentional, please call User.unset first.`);
        }
        this.userKVs = entity.userKVs;
        this.details = entity.details;
        this.permission = entity.permission;
        this.profile = entity.profile;
        this.sourceRef = entity.sourceRef;
        this.userName = entity.userName;
        this.isAuthenticated = true;
        return this;
    }
    /** {@inheritDoc User.unset} */
    unset() {
        this.userKVs = undefined;
        this.details = undefined;
        this.permission = undefined;
        this.profile = undefined;
        this.sourceRef = undefined;
        this.userName = undefined;
        this.isAuthenticated = false;
        return this;
    }
    /** @internal */
    userNameChanged() {
        Observable.notify(this, 'username');
    }
    /** {@inheritDoc User.username} */
    get username() {
        Observable.track(this, 'username');
        return this.userName;
    }
    /** {@inheritDoc User.isAnonymous} */
    get isAnonymous() {
        return !this.isAuthenticated;
    }
    /** {@inheritDoc User.isAdmin} */
    isAdmin() {
        return this.hasProfile('ADMIN');
    }
    /** {@inheritDoc User.isBanned} */
    isBanned() {
        return this.hasProfile('BANNED');
    }
    /** {@inheritDoc User.hasPermission} */
    hasPermission(permission) {
        return this.hasKey(this.permission, permission);
    }
    /** {@inheritDoc User.hasProfile} */
    hasProfile(profile) {
        return this.hasKey(this.profile, profile);
    }
    /** {@inheritDoc UserPathHistory.trackPath} */
    trackPath() {
        const { pathname, search, hash } = location;
        const path = `${pathname !== '/' ? pathname : ''}`;
        const urlPath = `${path}${search}${hash}`;
        if (this.config.trackPathExclusions.includes(urlPath)) {
            return;
        }
        this.pathHistory.push(urlPath);
        return urlPath;
    }
    /** {@inheritDoc UserPathHistory.clearPaths} */
    clearPaths() {
        this.pathHistory = [];
    }
    /** {@inheritDoc UserPathHistory.lastPath} */
    lastPath() {
        const length = this.pathHistory.length;
        return length > 0 ? this.pathHistory[length - 1] : undefined;
    }
    /** {@inheritDoc UserActionHistory.trackAction} */
    trackAction(details) {
        if (this.config.trackActionExclusions.includes(details)) {
            return;
        }
        this.actionHistory.push(details);
    }
    /** {@inheritDoc UserActionHistory.clearActions} */
    clearActions() {
        this.actionHistory = [];
    }
    /** {@inheritDoc UserActionHistory.lastAction} */
    lastAction() {
        const length = this.actionHistory.length;
        return length > 0 ? this.actionHistory[length - 1] : undefined;
    }
    /** @internal */
    hasKey(keys, key) {
        if (!keys || !keys.length || !key) {
            return false;
        }
        if (!Array.isArray(key)) {
            key = [key];
        }
        return key.some((k) => keys.includes(k));
    }
    /** {@inheritDoc User.addAuthenticationChangeListener} */
    addAuthenticationChangeListener(listener) {
        return this.authenticationChangeListeners.subscribe(listener);
    }
    /** @internal */
    isAuthenticatedChanged() {
        this.authenticationChangeListeners.publish(new CustomEvent(UserAuthenticationChangeType, {
            detail: this.isAuthenticated,
        }));
    }
}
__decorate([
    UserConfig
], DefaultUser.prototype, "config", void 0);
__decorate([
    observable
], DefaultUser.prototype, "pathHistory", void 0);
__decorate([
    observable
], DefaultUser.prototype, "actionHistory", void 0);
__decorate([
    observable
], DefaultUser.prototype, "isAuthenticated", void 0);
__decorate([
    observable
], DefaultUser.prototype, "userKVs", void 0);
__decorate([
    observable
], DefaultUser.prototype, "details", void 0);
__decorate([
    observable
], DefaultUser.prototype, "permission", void 0);
__decorate([
    observable
], DefaultUser.prototype, "profile", void 0);
__decorate([
    observable
], DefaultUser.prototype, "sourceRef", void 0);
__decorate([
    observable
], DefaultUser.prototype, "userName", void 0);
/**
 * User DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const User = DI.createInterface((x) => x.singleton(DefaultUser));
/**
 * Gets User from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getUser, User } from '@genesislcap/foundation-auth/user';
 * ...
 * private user: User = getUser();
 * ```
 *
 * @public
 */
export function getUser() {
    return DI.getOrCreateDOMContainer().get(User);
}
